@@ -1,6 +1,5 @@
 use std::ops::RangeInclusive;
 
-use async_trait::async_trait;
 use num_traits::Bounded;
 use num_traits::Zero;
 use thiserror::Error;
@@ -11,24 +10,41 @@ use crate::invocation::CommunicationErrorOf;
 use crate::invocation::Invocation;
 use crate::invocation::NayOf;
 use crate::invocation::RoundNumOf;
+use crate::retry::DoNotRetry;
+use crate::retry::RetryPolicy;
 
 pub struct AppendArgs<I: Invocation> {
     pub round: RangeInclusive<RoundNumOf<I>>,
     pub importance: Importance,
-    // TODO box internally
     pub retry_policy: Box<dyn RetryPolicy<Invocation = I> + Send>,
 }
 
-impl<C: Invocation> Default for AppendArgs<C> {
+impl<I: Invocation> Default for AppendArgs<I> {
     fn default() -> Self {
+        DoNotRetry::new().into()
+    }
+}
+
+impl<I, P> From<P> for AppendArgs<I>
+where
+    I: Invocation,
+    P: RetryPolicy<Invocation = I> + Send + 'static,
+{
+    fn from(retry_policy: P) -> Self {
         Self {
             round: Zero::zero()..=Bounded::max_value(),
             importance: Importance::GainLeadership,
-            retry_policy: Box::new(DoNotRetry::new()),
+            retry_policy: Box::new(retry_policy),
         }
     }
 }
 
+impl<I: Invocation> From<()> for AppendArgs<I> {
+    fn from(_: ()) -> Self {
+        Default::default()
+    }
+}
+
 /// Describes the importance of an append operation.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum Importance {
@@ -77,7 +93,11 @@ pub enum AppendError<I: Invocation> {
 
     /// The chosen round had already converged.
     #[error("round had already converged")]
-    Converged,
+    Converged {
+        /// Whether this node has already learned the converged on value for
+        /// this round.
+        caught_up: bool,
+    },
 
     /// Node does not currently know the shared state.
     #[error("node is disoriented")]
@@ -121,7 +141,10 @@ impl<I: Invocation> std::fmt::Debug for AppendError<I> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             AppendError::Aborted(err) => f.debug_tuple("AppendError::Aborted").field(err).finish(),
-            AppendError::Converged => f.debug_tuple("AppendError::Converged").finish(),
+            AppendError::Converged { caught_up } => f
+                .debug_struct("AppendError::Converged")
+                .field("caught_up", caught_up)
+                .finish(),
             AppendError::Disoriented => f.debug_tuple("AppendError::Disoriented").finish(),
             AppendError::Exiled => f.debug_tuple("AppendError::Exiled").finish(),
             AppendError::Lost => f.debug_tuple("AppendError::Lost").finish(),
@@ -142,33 +165,3 @@ impl<I: Invocation> std::fmt::Debug for AppendError<I> {
         }
     }
 }
-
-#[async_trait]
-pub trait RetryPolicy {
-    type Invocation: Invocation;
-
-    async fn eval(&mut self, err: AppendError<Self::Invocation>) -> Result<(), BoxError>;
-}
-
-#[derive(Clone, Copy, Debug)]
-pub struct DoNotRetry<I>(crate::util::PhantomSend<I>);
-
-impl<I> DoNotRetry<I> {
-    #[allow(clippy::new_without_default)]
-    pub fn new() -> Self {
-        Self(crate::util::PhantomSend::new())
-    }
-}
-
-#[async_trait]
-impl<I: Invocation> RetryPolicy for DoNotRetry<I> {
-    type Invocation = I;
-
-    async fn eval(&mut self, _err: AppendError<Self::Invocation>) -> Result<(), BoxError> {
-        Err(Box::new(AbortedError))
-    }
-}
-
-#[derive(Clone, Debug, Error)]
-#[error("append was aborted")]
-pub struct AbortedError;
