@@ -1,3 +1,4 @@
+use std::convert::Infallible;
 use std::sync::Arc;
 
 use thiserror::Error;
@@ -139,7 +140,9 @@ impl<I: Invocation> From<PrepareError<I>> for AppendError<I> {
                 rejections: Vec::new(),
             },
             PrepareError::Supplanted(_) => AppendError::Lost,
-            PrepareError::Converged(_, _) => AppendError::Converged,
+            PrepareError::Converged(_, log_entry) => AppendError::Converged {
+                caught_up: log_entry.is_some(),
+            },
             PrepareError::Passive => AppendError::Passive,
             PrepareError::ShutDown => AppendError::ShutDown,
         }
@@ -191,7 +194,9 @@ impl<I: Invocation> From<AcceptError<I>> for AppendError<I> {
     fn from(e: AcceptError<I>) -> Self {
         match e {
             AcceptError::Supplanted(_) => AppendError::Lost,
-            AcceptError::Converged(_, _) => AppendError::Converged,
+            AcceptError::Converged(_, log_entry) => AppendError::Converged {
+                caught_up: log_entry.is_some(),
+            },
             AcceptError::Passive => AppendError::Passive,
             AcceptError::Rejected(reason) => AppendError::NoQuorum {
                 abstentions: Vec::new(),
@@ -249,3 +254,9 @@ pub struct Disoriented;
 #[derive(Clone, Copy, Debug, Error)]
 #[error("node is shut down")]
 pub struct ShutDown;
+
+impl From<Infallible> for ShutDown {
+    fn from(_: Infallible) -> Self {
+        Self
+    }
+}
