@@ -32,12 +32,10 @@
 //! use std::collections::HashSet;
 //! # use std::convert::Infallible;
 //!
-//! # use async_trait::async_trait;
-//! # use futures::io::AsyncRead;
 //! use paxakos::{LogEntry, State};
 //! use uuid::Uuid;
 //!
-//! #[derive(Clone, Copy, Debug)]
+//! #[derive(Clone, Copy, Debug, serde::Deserialize, serde::Serialize)]
 //! pub enum CalcOp {
 //!     Add(f64, Uuid),
 //!     Div(f64, Uuid),
@@ -45,25 +43,9 @@
 //!     Sub(f64, Uuid),
 //! }
 //!
-//! # #[async_trait]
 //! impl LogEntry for CalcOp {
 //!     type Id = Uuid;
 //!
-//! #   type Reader = std::io::Cursor<Vec<u8>>;
-//! #   type ReadError = Infallible;
-//! #   
-//! #   async fn from_reader<R: AsyncRead + Send + Unpin>(_read: R) -> Result<Self, Self::ReadError> {
-//! #       unimplemented!()
-//! #   }
-//! #   
-//! #   fn size(&self) -> usize {
-//! #       unimplemented!()
-//! #   }
-//! #   
-//! #   fn to_reader(&self) -> Self::Reader {
-//! #       unimplemented!()
-//! #   }
-//! #
 //!     fn id(&self) -> Self::Id {
 //!         match self {
 //!             CalcOp::Add(_, id)
@@ -82,7 +64,6 @@
 //!     value: f64,
 //! }
 //!
-//! # #[async_trait]
 //! impl State for CalcState {
 //!     type Context = ();
 //!
@@ -90,23 +71,8 @@
 //!     type Outcome = f64;
 //!     type Event = f64;
 //!
-//! #   type Reader = std::io::Cursor<Vec<u8>>;
-//! #   type ReadError = Infallible;
-//! #   
 //! #   type Node = ();
 //! #   
-//! #   async fn from_reader<R: AsyncRead + Send + Unpin>(_read: R) -> Result<Self, Self::ReadError> {
-//! #       unimplemented!()
-//! #   }
-//! #   
-//! #   fn size(&self) -> usize {
-//! #       unimplemented!()
-//! #   }
-//! #   
-//! #   fn to_reader(&self) -> Self::Reader {
-//! #       unimplemented!()
-//! #   }
-//! #   
 //! #   fn cluster_at(&self, round_offset: std::num::NonZeroUsize) -> Vec<Self::Node> {
 //! #       vec![()]
 //! #   }
@@ -340,11 +306,6 @@
 //! of master leases. Assuming a scheme to delegate to the current master, this
 //! combination is highly efficient.
 //!
-//! ## Nightly Features
-//!
-//! Paxakos currently relies on several nightly features. None of them are
-//! crucial, but there hasn't been any need become compatible with stable Rust.
-//!
 //! # Future Direction
 //!
 //! This is a side project. I will work on the following as my time allows (in
@@ -364,16 +325,6 @@
 //! [State]: crate::state::State
 //! [State::cluster_at]: crate::state::State::cluster_at
 
-//
-// Nightly features
-#![feature(array_windows)]
-#![feature(btree_drain_filter)]
-#![feature(drain_filter)]
-#![feature(duration_zero)]
-#![feature(map_first_last)]
-#![feature(never_type)]
-#![feature(step_trait)]
-#![feature(with_options)]
 //
 // Lint configuration
 #![warn(rust_2018_idioms)]
@@ -381,20 +332,24 @@
 
 pub mod append;
 pub mod applicable;
+#[cfg(feature = "autofill")]
+pub mod autofill;
 pub mod cluster;
 pub mod communicator;
-pub mod deco;
+pub mod decoration;
 pub mod error;
 pub mod event;
+#[cfg(feature = "heartbeats")]
+pub mod heartbeats;
 pub mod invocation;
-#[cfg(feature = "master-leases")]
+pub mod leadership;
+#[cfg(feature = "leases")]
 pub mod leases;
-#[cfg(feature = "locking")]
-pub mod locking;
 mod log;
 pub mod node;
 #[cfg(feature = "prototyping")]
 pub mod prototyping;
+pub mod retry;
 pub mod state;
 #[cfg(feature = "tracer")]
 pub mod tracer;
@@ -407,9 +362,6 @@ use std::fmt::Debug;
 use std::hash::Hash;
 use std::sync::Arc;
 
-use serde::Deserialize;
-use serde::Serialize;
-
 // TODO move these three into the communicator module
 pub use error::AcceptError;
 pub use error::CommitError;
@@ -428,6 +380,7 @@ pub use node::NodeBuilder;
 pub use node::NodeHandle;
 pub use node::NodeInfo;
 pub use node::NodeKernel;
+pub use node::NodeKit;
 pub use node::NodeStatus;
 pub use node::RequestHandler;
 pub use node::Shutdown;
@@ -447,13 +400,13 @@ pub trait Number:
     + Hash
     + Debug
     + std::fmt::Display
-    + std::iter::Step
     + Into<u128>
     + TryFrom<u128>
     + TryFrom<usize>
     + TryInto<usize>
-    + serde::de::DeserializeOwned
-    + serde::Serialize
+where
+    Self: serde::Serialize,
+    for<'de> Self: serde::Deserialize<'de>,
 {
 }
 
@@ -469,7 +422,6 @@ impl<T> Number for T where
         + Hash
         + Debug
         + std::fmt::Display
-        + std::iter::Step
         + Into<u128>
         + TryFrom<u128>
         + TryFrom<usize>
@@ -500,7 +452,7 @@ pub trait Identifier: 'static + Copy + Debug + Eq + Hash + Send + Sync + Unpin {
 impl<T: 'static + Copy + Debug + Eq + Hash + Ord + Send + Sync + Unpin> Identifier for T {}
 
 /// A condition for a [Promise].
-#[derive(Clone, Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 pub struct Condition<R, C, E> {
     pub round_num: R,
     pub coord_num: C,
@@ -528,7 +480,7 @@ impl<R, C, E> From<(R, C, Arc<E>)> for Condition<R, C, E> {
 /// A promise not to accept certain proposals anymore.
 ///
 /// Please refer to the [description of the protocol](crate#protocol).
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 pub struct Promise<R, C, E>(Vec<Condition<R, C, E>>);
 
 impl<R: RoundNum, C, E> From<Vec<Condition<R, C, E>>> for Promise<R, C, E> {
@@ -633,7 +585,7 @@ impl<R: RoundNum, C: CoordNum, E> Promise<R, C, E> {
 /// Conflict to a prepare request or a proposal.
 ///
 /// Please refer to the [description of the protocol](crate#protocol).
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 pub enum Conflict<C, E> {
     Supplanted {
         coord_num: C,
