@@ -3,7 +3,7 @@ use futures::future::LocalBoxFuture;
 use futures::future::TryFutureExt;
 
 use crate::communicator::Communicator;
-use crate::deco::Decoration;
+use crate::decoration::Decoration;
 use crate::error::SpawnError;
 use crate::invocation::AbstainOf;
 use crate::invocation::CommunicationErrorOf;
@@ -19,7 +19,6 @@ use crate::invocation::StateOf;
 use crate::invocation::YeaOf;
 use crate::log::LogKeeping;
 use crate::node;
-use crate::node::Participation;
 #[cfg(feature = "tracer")]
 use crate::tracer::Tracer;
 use crate::voting::IndiscriminateVoter;
@@ -32,6 +31,7 @@ use super::snapshot::Snapshot;
 use super::CommunicatorOf;
 use super::InvocationOf;
 use super::NodeKernel;
+use super::NodeKit;
 use super::RequestHandlerFor;
 
 #[derive(Default)]
@@ -122,7 +122,7 @@ where
 {
     /// Starts the node without any state and in passive mode.
     pub fn without_state(self) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(None, Participation::Passive)
+        self.with_snapshot_and_passivity(None, false)
     }
 
     /// Starts a new cluster with the given initial state.
@@ -132,10 +132,7 @@ where
         self,
         initial_state: S,
     ) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(
-            initial_state.into().map(Snapshot::initial),
-            Participation::Active,
-        )
+        self.with_snapshot_and_passivity(initial_state.into().map(Snapshot::initial), false)
     }
 
     /// Resume operation from the given snapshot.
@@ -156,7 +153,7 @@ where
         self,
         snapshot: S,
     ) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(snapshot, Participation::Active)
+        self.with_snapshot_and_passivity(snapshot, false)
     }
 
     /// Resume operation from the given snapshot.
@@ -167,7 +164,7 @@ where
         self,
         snapshot: S,
     ) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(snapshot, Participation::Passive)
+        self.with_snapshot_and_passivity(snapshot, true)
     }
 
     /// Resume operation without a snapshot.
@@ -175,7 +172,7 @@ where
     /// The node will participate passively until it can be certain that it is
     /// not breaking any previous commitments.
     pub fn recovering_without_state(self) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(None, Participation::Passive)
+        self.with_snapshot_and_passivity(None, true)
     }
 
     /// Commence operation from the given snapshot.
@@ -196,7 +193,7 @@ where
         self,
         snapshot: S,
     ) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(snapshot, Participation::Active)
+        self.with_snapshot_and_passivity(snapshot, false)
     }
 
     /// Commence operation without a snapshot.
@@ -214,27 +211,24 @@ where
     /// [recovering_without]:
     /// NodeBuilderWithNodeIdAndWorkingDirAndCommunicator::recovering_without
     pub fn joining_without_state(self) -> NodeBuilder<NodeKernel<I, C>> {
-        self.with_snapshot_and_participation(None, Participation::Active)
+        self.with_snapshot_and_passivity(None, false)
     }
 
     #[doc(hidden)]
-    pub fn with_snapshot_and_participation<S: Into<Option<SnapshotFor<I>>>>(
+    pub fn with_snapshot_and_passivity<S: Into<Option<SnapshotFor<I>>>>(
         self,
         snapshot: S,
-        participation: Participation<RoundNumOf<I>>,
+        force_passive: bool,
     ) -> NodeBuilder<NodeKernel<I, C>> {
-        if matches!(participation, Participation::PartiallyActive(_)) {
-            panic!("Unsupported: {:?}", participation);
-        }
-
         let snapshot = snapshot.into();
 
         NodeBuilder {
+            kit: NodeKit::new(),
             working_dir: self.working_dir,
             node_id: self.node_id,
             communicator: self.communicator,
             snapshot,
-            participation,
+            force_passive,
             voter: IndiscriminateVoter::new(),
             log_keeping: Default::default(),
             finisher: Box::new(Ok),
@@ -249,12 +243,13 @@ type Finisher<N> =
     dyn FnOnce(NodeKernel<InvocationOf<N>, CommunicatorOf<N>>) -> Result<N, SpawnError>;
 
 pub struct NodeBuilder<N: Node, V = IndiscriminateVoterFor<N>> {
+    kit: NodeKit<InvocationOf<N>>,
     working_dir: Option<std::path::PathBuf>,
     node_id: node::NodeIdOf<N>,
     voter: V,
     communicator: CommunicatorOf<N>,
     snapshot: Option<node::SnapshotFor<N>>,
-    participation: Participation<node::RoundNumOf<N>>,
+    force_passive: bool,
     log_keeping: LogKeeping,
     finisher: Box<Finisher<N>>,
 
@@ -262,18 +257,16 @@ pub struct NodeBuilder<N: Node, V = IndiscriminateVoterFor<N>> {
     tracer: Option<Box<dyn Tracer<InvocationOf<N>>>>,
 }
 
-impl<I, C, V> NodeBuilder<NodeKernel<I, C>, V>
+impl<N, V> NodeBuilder<N, V>
 where
-    I: Invocation,
-    C: Communicator<
-        Node = NodeOf<I>,
-        RoundNum = RoundNumOf<I>,
-        CoordNum = CoordNumOf<I>,
-        LogEntry = LogEntryOf<I>,
-        Error = CommunicationErrorOf<I>,
-        Yea = YeaOf<I>,
-        Nay = NayOf<I>,
-        Abstain = AbstainOf<I>,
+    N: Node + 'static,
+    V: Voter<
+        State = node::StateOf<N>,
+        RoundNum = node::RoundNumOf<N>,
+        CoordNum = node::CoordNumOf<N>,
+        Abstain = node::AbstainOf<N>,
+        Yea = node::YeaOf<N>,
+        Nay = node::NayOf<N>,
     >,
 {
     pub fn limiting_applied_entry_logs_to(mut self, limit: usize) -> Self {
@@ -291,20 +284,21 @@ where
     }
 
     #[cfg(feature = "tracer")]
-    pub fn traced_by<T: Into<Box<dyn Tracer<I>>>>(mut self, tracer: T) -> Self {
+    pub fn traced_by<T: Into<Box<dyn Tracer<InvocationOf<N>>>>>(mut self, tracer: T) -> Self {
         self.tracer = Some(tracer.into());
 
         self
     }
 
-    pub fn voting_with<T>(self, voter: T) -> NodeBuilder<NodeKernel<I, C>, T> {
+    pub fn voting_with<T>(self, voter: T) -> NodeBuilder<N, T> {
         NodeBuilder {
+            kit: self.kit,
             working_dir: self.working_dir,
             node_id: self.node_id,
             voter,
             communicator: self.communicator,
             snapshot: self.snapshot,
-            participation: self.participation,
+            force_passive: self.force_passive,
             log_keeping: self.log_keeping,
             finisher: self.finisher,
 
@@ -312,20 +306,13 @@ where
             tracer: self.tracer,
         }
     }
-}
 
-impl<N, V> NodeBuilder<N, V>
-where
-    N: Node + 'static,
-    V: Voter<
-        State = node::StateOf<N>,
-        RoundNum = node::RoundNumOf<N>,
-        CoordNum = node::CoordNumOf<N>,
-        Abstain = node::AbstainOf<N>,
-        Yea = node::YeaOf<N>,
-        Nay = node::NayOf<N>,
-    >,
-{
+    pub fn using(mut self, kit: NodeKit<InvocationOf<N>>) -> Self {
+        self.kit = kit;
+
+        self
+    }
+
     pub fn decorated_with<D>(self, arguments: <D as Decoration>::Arguments) -> NodeBuilder<D, V>
     where
         D: Decoration<
@@ -337,11 +324,12 @@ where
         let finisher = self.finisher;
 
         NodeBuilder {
+            kit: self.kit,
             working_dir: self.working_dir,
             node_id: self.node_id,
             communicator: self.communicator,
             snapshot: self.snapshot,
-            participation: self.participation,
+            force_passive: self.force_passive,
             voter: self.voter,
             log_keeping: self.log_keeping,
             finisher: Box::new(move |x| ((finisher)(x)).and_then(|node| D::wrap(node, arguments))),
@@ -365,6 +353,7 @@ where
         let finisher = self.finisher;
 
         NodeKernel::spawn(
+            self.kit,
             self.node_id,
             self.communicator,
             super::SpawnArgs {
@@ -373,7 +362,7 @@ where
                 node_id: self.node_id,
                 voter: self.voter,
                 snapshot: self.snapshot,
-                participation: self.participation,
+                force_passive: self.force_passive,
                 log_keeping: self.log_keeping,
                 #[cfg(feature = "tracer")]
                 tracer: self.tracer,
