@@ -12,6 +12,7 @@ mod status;
 
 use std::sync::Arc;
 
+use futures::channel::mpsc;
 use futures::future::BoxFuture;
 use futures::future::LocalBoxFuture;
 
@@ -38,6 +39,8 @@ pub use shutdown::Shutdown;
 pub use snapshot::Snapshot;
 pub use status::NodeStatus;
 
+use state_keeper::StateKeeperKit;
+
 pub type AbstainOf<N> = invocation::AbstainOf<InvocationOf<N>>;
 pub type CommunicationErrorOf<N> = invocation::CommunicationErrorOf<InvocationOf<N>>;
 pub type CommunicatorOf<N> = <N as Node>::Communicator;
@@ -114,10 +117,10 @@ pub trait Node: Sized {
 
     fn read_stale(&self) -> LocalBoxFuture<'_, Result<Arc<StateOf<Self>>, Disoriented>>;
 
-    fn append<A: ApplicableTo<StateOf<Self>> + 'static>(
+    fn append<A: ApplicableTo<StateOf<Self>> + 'static, P: Into<AppendArgs<Self::Invocation>>>(
         &self,
         applicable: A,
-        args: AppendArgs<Self::Invocation>,
+        args: P,
     ) -> LocalBoxFuture<'static, AppendResultFor<Self, A>>;
 
     fn shut_down(self) -> Self::Shutdown;
@@ -160,7 +163,7 @@ pub struct SpawnArgs<I: Invocation, V> {
     pub node_id: invocation::NodeIdOf<I>,
     pub voter: V,
     pub snapshot: Option<invocation::SnapshotFor<I>>,
-    pub participation: Participation<invocation::RoundNumOf<I>>,
+    pub force_passive: bool,
     pub log_keeping: LogKeeping,
     #[cfg(feature = "tracer")]
     pub tracer: Option<Box<dyn Tracer<I>>>,
@@ -198,3 +201,32 @@ pub enum Participation<R> {
     /// and the node transitions into `PartiallyActive` participation.
     Passive,
 }
+
+/// Allows getting a `NodeHandle` before the `Node` itself is built.
+pub struct NodeKit<I: Invocation> {
+    state_keeper: StateKeeperKit<I>,
+    sender: mpsc::Sender<handle::RequestAndResponseSender<I>>,
+    receiver: mpsc::Receiver<handle::RequestAndResponseSender<I>>,
+}
+
+impl<I: Invocation> NodeKit<I> {
+    pub fn new() -> Self {
+        let (sender, receiver) = mpsc::channel(16);
+
+        Self {
+            state_keeper: StateKeeperKit::new(),
+            sender,
+            receiver,
+        }
+    }
+
+    pub fn handle(&self) -> NodeHandle<I> {
+        NodeHandle::new(self.sender.clone(), self.state_keeper.handle())
+    }
+}
+
+impl<I: Invocation> Default for NodeKit<I> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
