@@ -1,57 +1,82 @@
 use std::collections::BTreeMap;
-use std::convert::TryInto;
-use std::io::Read;
+use std::collections::HashSet;
 use std::sync::Arc;
 
-use futures::io;
-use futures::io::AsyncRead;
-use futures::io::AsyncReadExt;
 use num_traits::One;
 use num_traits::Zero;
-use serde::Deserialize;
-use serde::Serialize;
 
 use crate::state::LogEntryOf;
 use crate::state::State;
 use crate::CoordNum;
 use crate::RoundNum;
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 #[serde(bound(
-    serialize = "S: Serialize, S::LogEntry: Serialize",
-    deserialize = "S: Deserialize<'de>, S::LogEntry: Deserialize<'de>"
+    serialize = "S: serde::Serialize",
+    deserialize = "S: serde::Deserialize<'de>"
 ))]
 pub struct Snapshot<S: State, R: RoundNum, C: CoordNum> {
     identity: Arc<()>,
     inner: Arc<SnapshotInner<S, R, C>>,
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 #[serde(bound(
-    serialize = "S: Serialize, S::LogEntry: Serialize",
-    deserialize = "S: Deserialize<'de>, S::LogEntry: Deserialize<'de>"
+    serialize = "S: serde::Serialize",
+    deserialize = "S: serde::Deserialize<'de>"
 ))]
 struct SnapshotInner<S: State, R: RoundNum, C: CoordNum> {
     meta_state: MetaState<S, R, C>,
     state: Arc<S>,
 }
 
+/// See [super::Participation].
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
+#[serde(bound(deserialize = ""))] // *sadface*
+pub(crate) enum Participation<R: RoundNum, C: CoordNum> {
+    Active,
+    PartiallyActive(R),
+    Passive { observed_proposals: HashSet<(R, C)> },
+}
+
+impl<'a, R: RoundNum, C: CoordNum> From<&'a Participation<R, C>> for super::Participation<R> {
+    fn from(p: &'a Participation<R, C>) -> Self {
+        match p {
+            Participation::Active => super::Participation::Active,
+            Participation::PartiallyActive(r) => super::Participation::PartiallyActive(*r),
+            Participation::Passive { .. } => super::Participation::Passive,
+        }
+    }
+}
+
+impl<R: RoundNum, C: CoordNum> Participation<R, C> {
+    pub(crate) fn passive() -> Self {
+        Self::Passive {
+            observed_proposals: HashSet::new(),
+        }
+    }
+}
+
 pub(crate) struct DeconstructedSnapshot<S: State, R: RoundNum, C: CoordNum> {
     pub(crate) state_round: R,
     pub(crate) state: Arc<S>,
+    pub(crate) highest_observed_round_num: Option<R>,
     pub(crate) highest_observed_coord_num: C,
+    pub(crate) participation: Participation<R, C>,
     pub(crate) promises: BTreeMap<R, C>,
     pub(crate) accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, serde::Deserialize, serde::Serialize)]
 #[serde(bound(
-    serialize = "S::LogEntry: Serialize",
-    deserialize = "S::LogEntry: Deserialize<'de>"
+    serialize = "S: serde::Serialize",
+    deserialize = "S: serde::Deserialize<'de>"
 ))]
 struct MetaState<S: State, R: RoundNum, C: CoordNum> {
     state_round: R,
+    highest_observed_round_num: Option<R>,
     highest_observed_coord_num: C,
+    participation: Participation<R, C>,
     promises: BTreeMap<R, C>,
     accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
 }
@@ -60,7 +85,9 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
     pub(crate) fn new(
         round: R,
         state: Arc<S>,
+        highest_observed_round_num: Option<R>,
         highest_observed_coord_num: C,
+        participation: Participation<R, C>,
         promises: BTreeMap<R, C>,
         accepted_entries: BTreeMap<R, (C, Arc<LogEntryOf<S>>)>,
     ) -> Self {
@@ -68,10 +95,12 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
             identity: Arc::new(()),
             inner: Arc::new(SnapshotInner {
                 meta_state: MetaState {
+                    state_round: round,
+                    highest_observed_round_num,
+                    highest_observed_coord_num,
+                    participation,
                     promises,
                     accepted_entries,
-                    state_round: round,
-                    highest_observed_coord_num,
                 },
                 state,
             }),
@@ -85,53 +114,14 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
         Self::new(
             Zero::zero(),
             state.into(),
+            None,
             One::one(),
+            Participation::Active,
             promises,
             BTreeMap::new(),
         )
     }
 
-    pub async fn from_reader<T: AsyncRead + Send + Unpin>(
-        mut read: T,
-    ) -> Result<Self, crate::error::BoxError>
-    where
-        LogEntryOf<S>: serde::de::DeserializeOwned,
-    {
-        let mut meta_state_len = [0; 64 / 8];
-        read.read_exact(&mut meta_state_len).await?;
-        let meta_state_len = u64::from_be_bytes(meta_state_len);
-
-        let mut meta_state = vec![0; meta_state_len.try_into().unwrap()];
-        read.read_exact(&mut meta_state).await?;
-
-        let meta_state: MetaState<S, R, C> = bincode::deserialize(&meta_state)?;
-
-        let state = <S as State>::from_reader(read).await?;
-
-        Ok(Self {
-            identity: Arc::new(()),
-            inner: Arc::new(SnapshotInner {
-                meta_state,
-                state: Arc::new(state),
-            }),
-        })
-    }
-
-    pub fn to_reader(&self) -> SnapshotReader<S>
-    where
-        LogEntryOf<S>: Serialize,
-    {
-        // TODO eliminate bincode dependency
-        let meta_state = bincode::serialize(&self.inner.meta_state).unwrap();
-        let meta_state_len: u64 = meta_state.len().try_into().unwrap();
-        let meta_state_len = meta_state_len.to_be_bytes();
-        let meta_state_len = std::io::Cursor::new(meta_state_len);
-        let meta_state = std::io::Cursor::new(meta_state);
-        let state = self.inner.state.to_reader();
-
-        SnapshotReader(meta_state_len.chain(meta_state.chain(state)))
-    }
-
     pub fn round(&self) -> R {
         self.inner.meta_state.state_round
     }
@@ -145,29 +135,20 @@ impl<S: State, R: RoundNum, C: CoordNum> Snapshot<S, R, C> {
             .map(|inner| DeconstructedSnapshot {
                 state_round: inner.meta_state.state_round,
                 state: inner.state,
+                highest_observed_round_num: inner.meta_state.highest_observed_round_num,
                 highest_observed_coord_num: inner.meta_state.highest_observed_coord_num,
+                participation: inner.meta_state.participation,
                 promises: inner.meta_state.promises,
                 accepted_entries: inner.meta_state.accepted_entries,
             })
             .unwrap_or_else(|s| DeconstructedSnapshot {
                 state_round: s.meta_state.state_round,
                 state: Arc::clone(&s.state),
+                highest_observed_round_num: s.meta_state.highest_observed_round_num,
                 highest_observed_coord_num: s.meta_state.highest_observed_coord_num,
+                participation: s.meta_state.participation.clone(),
                 promises: s.meta_state.promises.clone(),
                 accepted_entries: s.meta_state.accepted_entries.clone(),
             })
     }
 }
-
-type MetaStateLenAndMetaStateAndState<S> =
-    std::io::Chain<std::io::Cursor<[u8; 64 / 8]>, MetaStateAndState<S>>;
-
-type MetaStateAndState<S> = std::io::Chain<std::io::Cursor<Vec<u8>>, <S as State>::Reader>;
-
-pub struct SnapshotReader<S: State>(MetaStateLenAndMetaStateAndState<S>);
-
-impl<S: State> Read for SnapshotReader<S> {
-    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
-        self.0.read(buf)
-    }
-}
