@@ -54,9 +54,11 @@ use crate::invocation::StateOf;
 use crate::invocation::YeaOf;
 use crate::log::LogEntry;
 use crate::node::NodeInfo;
+use crate::state;
 use crate::state::State;
 #[cfg(feature = "tracer")]
 use crate::tracer::Tracer;
+use crate::util::NumberIter;
 use crate::voting;
 use crate::voting::Voter;
 use crate::Condition;
@@ -65,6 +67,7 @@ use crate::Promise;
 use crate::RoundNum;
 
 use super::snapshot::DeconstructedSnapshot;
+use super::snapshot::Participation;
 use super::snapshot::Snapshot;
 use super::status::NodeStatus;
 use super::SpawnArgs;
@@ -92,31 +95,6 @@ macro_rules! assert_unreachable {
     }};
 }
 
-#[derive(Debug)]
-enum Participation<R, C> {
-    Active,
-    PartiallyActive(R),
-    Passive { observed_proposals: HashSet<(R, C)> },
-}
-
-impl<R, C> Participation<R, C> {
-    pub fn passive() -> Self {
-        Self::Passive {
-            observed_proposals: HashSet::new(),
-        }
-    }
-}
-
-impl<R, C> From<super::Participation<R>> for Participation<R, C> {
-    fn from(p: super::Participation<R>) -> Self {
-        match p {
-            super::Participation::Active => Self::Active,
-            super::Participation::Passive => Self::passive(),
-            super::Participation::PartiallyActive(_) => unreachable!(),
-        }
-    }
-}
-
 enum AcceptPolicy<I: Invocation> {
     Irrejectable,
     Rejectable(Option<NodeOf<I>>),
@@ -127,6 +105,68 @@ type RoundNumRequest<I> = (RangeInclusive<RoundNumOf<I>>, ResponseSender<I>);
 type AcceptedEntry<I> = (CoordNumOf<I>, Arc<LogEntryOf<I>>);
 type PendingCommit<I> = (Instant, CoordNumOf<I>, Arc<LogEntryOf<I>>);
 
+struct Init<S: State, R: RoundNum, C: CoordNum> {
+    state_round: R,
+    state: Option<Arc<S>>,
+    highest_observed_round_num: Option<R>,
+    highest_observed_coord_num: C,
+    participation: Participation<R, C>,
+    promises: BTreeMap<R, C>,
+    accepted_entries: BTreeMap<R, (C, Arc<state::LogEntryOf<S>>)>,
+}
+
+impl<S: State, R: RoundNum, C: CoordNum> Default for Init<S, R, C> {
+    fn default() -> Self {
+        Self {
+            state_round: Zero::zero(),
+            state: None,
+            highest_observed_round_num: None,
+            highest_observed_coord_num: One::one(),
+            participation: Participation::Active,
+            promises: {
+                let mut promises = BTreeMap::new();
+                promises.insert(Zero::zero(), One::one());
+                promises
+            },
+            accepted_entries: BTreeMap::new(),
+        }
+    }
+}
+
+impl<S: State, R: RoundNum, C: CoordNum> From<DeconstructedSnapshot<S, R, C>> for Init<S, R, C> {
+    fn from(s: DeconstructedSnapshot<S, R, C>) -> Self {
+        Self {
+            state_round: s.state_round,
+            state: Some(s.state),
+            highest_observed_round_num: s.highest_observed_round_num,
+            highest_observed_coord_num: s.highest_observed_coord_num,
+            participation: s.participation,
+            promises: s.promises,
+            accepted_entries: s.accepted_entries,
+        }
+    }
+}
+
+pub struct StateKeeperKit<I: Invocation> {
+    handle: StateKeeperHandle<I>,
+    receiver: mpsc::Receiver<RequestAndResponseSender<I>>,
+}
+
+impl<I: Invocation> StateKeeperKit<I> {
+    pub fn new() -> Self {
+        let (sender, receiver) = mpsc::channel(32);
+
+        Self {
+            handle: StateKeeperHandle::new(sender),
+            receiver,
+        }
+    }
+
+    pub fn handle(&self) -> StateKeeperHandle<I> {
+        self.handle.clone()
+    }
+}
+
 pub struct StateKeeper<I, V>
 where
     I: Invocation,
@@ -150,10 +190,12 @@ where
     release_sender: mpsc::UnboundedSender<Release<RoundNumOf<I>>>,
     release_receiver: mpsc::UnboundedReceiver<Release<RoundNumOf<I>>>,
 
-    // TODO persist in snapshots
+    /// The highest _observed_ round number so far.
+    ///
+    /// This number is used to determine whether this node is lagging.
     highest_observed_round_num: Option<RoundNumOf<I>>,
 
-    /// The highest coordination _observed_ number so far.
+    /// The highest _observed_  coordination number so far.
     ///
     /// Naively it might make sense to track these in a BTreeMap, similar to
     /// promises. However, the highest observed coordination number is only used
@@ -206,7 +248,6 @@ where
     pending_commits: BTreeMap<RoundNumOf<I>, PendingCommit<I>>,
     awaiters: HashMap<LogEntryIdOf<I>, Vec<Awaiter<I>>>,
 
-    // TODO persist in snapshots
     participation: Participation<RoundNumOf<I>, CoordNumOf<I>>,
     /// last status that was observable from the outside
     status: NodeStatus,
@@ -234,26 +275,24 @@ where
     >,
 {
     pub async fn spawn(
+        kit: StateKeeperKit<I>,
         args: SpawnArgs<I, V>,
     ) -> Result<
         (
             NodeStatus,
             super::Participation<RoundNumOf<I>>,
             EventStream<I>,
-            StateKeeperHandle<I>,
             ProofOfLife,
         ),
         SpawnError,
     > {
-        let (req_send, req_recv) = mpsc::channel(32);
-
         let (evt_send, evt_recv) = mpsc::channel(32);
 
         let proof_of_life = ProofOfLife::new();
 
         let (send, recv) = oneshot::channel();
 
-        Self::start_and_run_new(args, send, req_recv, evt_send);
+        Self::start_and_run_new(args, send, kit.receiver, evt_send);
 
         let start_result = recv.await.expect("StateKeeper failed to start");
 
@@ -262,7 +301,6 @@ where
                 initial_status,
                 initial_participation,
                 EventStream { delegate: evt_recv },
-                StateKeeperHandle::new(req_send),
                 proof_of_life,
             )
         })
@@ -280,7 +318,7 @@ where
             let node_id = spawn_args.node_id;
             let voter = spawn_args.voter;
             let snapshot = spawn_args.snapshot;
-            let participation = spawn_args.participation;
+            let force_passive = spawn_args.force_passive;
             let log_keeping = spawn_args.log_keeping;
             #[cfg(feature = "tracer")]
             let tracer = spawn_args.tracer;
@@ -291,38 +329,23 @@ where
                 .map(|_| NodeStatus::Lagging)
                 .unwrap_or(NodeStatus::Disoriented);
 
-            let (state_round, state, highest_observed_coord_num, promises, accepted_entries) =
-                snapshot
-                    .map(|s| {
-                        let DeconstructedSnapshot {
-                            state_round,
-                            state,
-                            highest_observed_coord_num,
-                            promises,
-                            accepted_entries,
-                        } = s.deconstruct();
+            let Init {
+                state_round,
+                state,
+                highest_observed_round_num,
+                highest_observed_coord_num,
+                participation,
+                promises,
+                accepted_entries,
+            } = snapshot
+                .map(|s| Init::from(s.deconstruct()))
+                .unwrap_or_default();
 
-                        (
-                            state_round,
-                            Some(state),
-                            highest_observed_coord_num,
-                            promises,
-                            accepted_entries,
-                        )
-                    })
-                    .unwrap_or_else(|| {
-                        (
-                            Zero::zero(),
-                            None,
-                            One::one(),
-                            {
-                                let mut promises = BTreeMap::new();
-                                promises.insert(Zero::zero(), One::one());
-                                promises
-                            },
-                            BTreeMap::new(),
-                        )
-                    });
+            let participation = if force_passive {
+                Participation::passive()
+            } else {
+                participation
+            };
 
             let (start_result, working_dir) = match working_dir {
                 Some(working_dir) => match WorkingDir::init(working_dir, log_keeping) {
@@ -333,7 +356,9 @@ where
             };
 
             let failed = start_result.is_err();
-            let _ = start_result_sender.send(start_result.map(|_| (initial_status, participation)));
+            let _ = start_result_sender.send(
+                start_result.map(|_| (initial_status, super::Participation::from(&participation))),
+            );
 
             if failed {
                 return;
@@ -363,13 +388,13 @@ where
                 round_num_requests: VecDeque::new(),
                 acquired_round_nums: HashSet::new(),
 
-                highest_observed_round_num: None,
+                highest_observed_round_num,
                 highest_observed_coord_num,
 
                 pending_commits: BTreeMap::new(),
                 awaiters: HashMap::new(),
 
-                participation: participation.into(),
+                participation,
                 status: initial_status,
 
                 queued_events: VecDeque::new(),
@@ -480,7 +505,7 @@ where
                     let range =
                         *range.start()..=std::cmp::min(round + concurrency_bound, *range.end());
 
-                    for round_num in range {
+                    for round_num in NumberIter::from_range(range) {
                         if !acquireds.contains(&round_num) && !deferreds.contains_key(&round_num) {
                             acquireds.insert(round_num);
 
@@ -849,7 +874,7 @@ where
             if round_num > self.state_round
                 && round_num <= self.state_round + into_round_num(state.concurrency())
             {
-                // FIXME lots of duplication with NodeInner::determine_coord_num
+                // TODO lots of duplication with NodeInner::determine_coord_num
                 let round_offset = crate::util::usize_delta(round_num, self.state_round);
                 let round_offset = std::num::NonZeroUsize::new(round_offset)
                     .expect("Zero was ruled out via equality check");
@@ -895,7 +920,9 @@ where
         Ok(Snapshot::new(
             self.state_round,
             Arc::clone(state),
+            self.highest_observed_round_num,
             self.highest_observed_coord_num,
+            self.participation.clone(),
             self.promises.clone(),
             self.accepted_entries.clone(),
         ))
@@ -914,15 +941,18 @@ where
         let DeconstructedSnapshot {
             state_round,
             state,
+            highest_observed_round_num,
             highest_observed_coord_num,
             promises,
             accepted_entries,
+            ..
         } = snapshot.deconstruct();
 
         self.become_passive();
 
         self.state_round = state_round;
         self.state = Some(Arc::clone(&state));
+        self.highest_observed_round_num = highest_observed_round_num;
         self.highest_observed_coord_num = highest_observed_coord_num;
         self.promises = promises;
         self.accepted_entries = accepted_entries;
@@ -993,7 +1023,7 @@ where
         //
         // What we give up here is the ability for a node to become leader for rounds
         // `r1..r2` while another node _remains_ leader for rounds `r2..`.
-        let strongest_promise = *self.promises.last_key_value().expect("last promise").1;
+        let strongest_promise = *self.promises.values().rev().next().expect("last promise");
 
         // Paxos requires that we reject requests whose coordination number is less than
         // or equal to the one of the previously accepted proposal. Because of how we
@@ -1375,9 +1405,9 @@ where
         );
         overriding_insert(&mut self.promises, Zero::zero(), first_non_obsolete_promise);
 
-        while let Some(first_accepted_entry) = self.accepted_entries.first_entry() {
-            if *first_accepted_entry.key() <= self.state_round {
-                first_accepted_entry.remove();
+        while let Some(first_accepted_key) = self.accepted_entries.keys().next().copied() {
+            if first_accepted_key <= self.state_round {
+                self.accepted_entries.remove(&first_accepted_key);
             } else {
                 break;
             }
@@ -1387,7 +1417,7 @@ where
             .awaiters
             .iter_mut()
             .filter_map(|(eid, senders)| {
-                senders.drain_filter(|sender| sender.is_canceled());
+                senders.retain(|sender| !sender.is_canceled());
 
                 if senders.is_empty() {
                     Some(*eid)
@@ -1423,7 +1453,7 @@ fn overriding_insert<R: RoundNum, C: CoordNum>(
     round_num: R,
     coord_num: C,
 ) {
-    map.drain_filter(|r, c| *r > round_num && *c <= coord_num);
+    map.retain(|r, c| *r <= round_num || *c > coord_num);
     map.insert(round_num, coord_num);
 }
 
