@@ -250,9 +250,10 @@ impl<R: RoundNum> WorkingDir<R> {
             while !self.applied_entry_logs.is_empty()
                 && self.applied_entry_logs.len() >= self.log_keeping.logs_kept
             {
-                let (_handle, (path, _file)) = self
+                let first_key = *self.applied_entry_logs.keys().next().unwrap();
+                let (path, _file) = self
                     .applied_entry_logs
-                    .pop_first()
+                    .remove(&first_key)
                     .expect("oldest applied log entry");
 
                 if let Err(err) = fs::remove_file(&path) {
@@ -316,7 +317,9 @@ impl<R: RoundNum> WorkingDir<R> {
         let offset = io::seek_in(path, file, SeekFrom::End(0))?;
 
         io::write_u32_to(path, file, APPLY)?;
-        io::write_usize_as_u32_to(path, file, log_entry.size())?;
+        let serialized = rmp_serde::to_vec_named(&log_entry)
+            .map_err(|err| IoError::invalid_data("Could not serialize log entry.", err))?;
+        io::write_usize_as_u32_to(path, file, serialized.len())?;
 
         let mut checksumming_write = io::Checksumming::from(file);
 
@@ -325,7 +328,7 @@ impl<R: RoundNum> WorkingDir<R> {
 
         io::copy_from_to(
             format!("log-entry://{:?}", log_entry.id()),
-            &mut log_entry.to_reader(),
+            &mut std::io::Cursor::new(serialized),
             path.display(),
             &mut checksumming_write,
         )?;
@@ -343,11 +346,11 @@ impl<R: RoundNum> WorkingDir<R> {
         let mut path = self.path.clone();
         path.push(format!("applied_entry.log.{}", Base62Of(round_num.into())));
 
-        let mut file = fs::File::with_options()
-            .create(true)
-            .write(true)
-            .open(&path)
-            .map_err(io::to_failed_to_open(&path))?;
+        let mut options = fs::OpenOptions::new();
+        options.create(true);
+        options.write(true);
+
+        let mut file = options.open(&path).map_err(io::to_failed_to_open(&path))?;
 
         io::write_u32_to(&path, &mut file, MAGIC_BYTES)?;
 
@@ -374,7 +377,7 @@ impl<R: RoundNum> WorkingDir<R> {
 
         let (path, file) = self
             .applied_entry_logs
-            .get_mut(&log_handle)
+            .get_mut(log_handle)
             .expect("applied entry log given handle");
 
         match Self::read_applied_entry(path, file, *offset) {
@@ -410,8 +413,7 @@ impl<R: RoundNum> WorkingDir<R> {
         })?;
         let coord_num = io::recover_coord_num(coord_num)?;
         let log_entry = io::recover_log_entry(
-            futures::executor::block_on(E::from_reader(futures::io::AllowStdIo::new(file)))
-                .map_err::<BoxError, _>(|e| Box::new(e)),
+            rmp_serde::from_read(file).map_err::<BoxError, _>(|e| Box::new(e)),
         )?;
 
         Ok((coord_num, Arc::new(log_entry)))
